@@ -53,6 +53,11 @@ Module for describing beamline/experiment specific data recipes.
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
+import numpy as np
+import os.path
+import re
+import logging
+import dxchange.reader as dxreader
 
 __authors__ = "Doga Gursoy, Luis Barroso-Luque, Francesco De Carlo"
 __copyright__ = "Copyright (c) 2015-2016, UChicago Argonne, LLC."
@@ -64,6 +69,7 @@ __all__ = ['read_als_832',
            'read_aps_1id',
            'read_aps_2bm',
            'read_aps_7bm',
+           'read_aps_8bm',
            'read_aps_13bm',
            'read_aps_13id',
            'read_aps_26id',
@@ -76,17 +82,10 @@ __all__ = ['read_als_832',
            'read_petraIII_p05',
            'read_sls_tomcat']
 
-
-import numpy as np
-import os.path
-import re
-import logging
-import dxchange.reader as dxreader
-
 logger = logging.getLogger(__name__)
 
 
-def read_als_832(fname, ind_tomo=None, normalized=False, sino=None):
+def read_als_832(fname, ind_tomo=None, normalized=False, proj=None, sino=None):
     """
     Read ALS 8.3.2 standard data format.
 
@@ -98,12 +97,15 @@ def read_als_832(fname, ind_tomo=None, normalized=False, sino=None):
     ind_tomo : list of int, optional
         Indices of the projection files to read.
 
-    normalized : boolean
+    normalized : boolean, optional
         If False, darks and flats will not be read. This should
         only be used for cases where tomo is already normalized.
         8.3.2 has a plugin that normalization is preferred to be
         done with prior to tomopy reconstruction.
 
+    proj : {sequence, int}, optional
+        Specify projections to read. (start, end, step)
+
     sino : {sequence, int}, optional
         Specify sinograms to read. (start, end, step)
 
@@ -150,6 +152,8 @@ def read_als_832(fname, ind_tomo=None, normalized=False, sino=None):
     contents.close()
     if ind_tomo is None:
         ind_tomo = list(range(0, nproj))
+    if proj is not None:
+        ind_tomo = ind_tomo[slice(*proj)]
     if not normalized:
         ind_flat = list(range(0, nflat))
         if inter_bright > 0:
@@ -292,6 +296,8 @@ def read_als_832h5(fname, ind_tomo=None, ind_flat=None, ind_dark=None,
         # Create arrays of indices to read projections, flats and darks
         if ind_tomo is None:
             ind_tomo = list(range(0, nproj))
+        if proj is not None:
+            ind_tomo = ind_tomo[slice(*proj)]
         ind_dark = list(range(0, ndark))
         group_dark = [nproj - 1]
         ind_flat = list(range(0, nflat))
@@ -306,15 +312,15 @@ def read_als_832h5(fname, ind_tomo=None, ind_flat=None, ind_dark=None,
             group_flat = None
 
         tomo = dxreader.read_hdf5_stack(
-            dgroup, tomo_name, ind_tomo, slc=(proj, sino))
+            dgroup, tomo_name, ind_tomo, slc=(None, sino))
 
-        flat = dxreader.read_hdf5_stack(dgroup, flat_name, ind_flat, slc=(None, sino),
-                                        out_ind=group_flat)
+        flat = dxreader.read_hdf5_stack(
+            dgroup, flat_name, ind_flat, slc=(None, sino), out_ind=group_flat)
 
-        dark = dxreader.read_hdf5_stack(dgroup, dark_name, ind_dark, slc=(None, sino),
-                                        out_ind=group_dark)
+        dark = dxreader.read_hdf5_stack(
+            dgroup, dark_name, ind_dark, slc=(None, sino), out_ind=group_dark)
 
-    return tomo, flat, dark
+    return tomo, flat, dark, dxreader._map_loc(ind_tomo, group_flat)
 
 
 def read_anka_topotomo(
@@ -327,10 +333,10 @@ def read_anka_topotomo(
     fname : str
         Path to data folder name without indices and extension.
 
-    ind_tomo : list of int, optional
+    ind_tomo : list of int
         Indices of the projection files to read.
 
-    ind_flat : list of int, optional
+    ind_flat : list of int
         Indices of the flat field files to read.
 
     ind_dark : list of int, optional
@@ -357,12 +363,14 @@ def read_anka_topotomo(
     tomo_name = os.path.join(fname, 'radios', 'image_00000.tif')
     flat_name = os.path.join(fname, 'flats', 'image_00000.tif')
     dark_name = os.path.join(fname, 'darks', 'image_00000.tif')
+    if proj is not None:
+        ind_tomo = ind_tomo[slice(*proj)]
     tomo = dxreader.read_tiff_stack(
-        tomo_name, ind=ind_tomo, digit=5, slc=(sino, proj))
+        tomo_name, ind=ind_tomo, slc=(sino, None))
     flat = dxreader.read_tiff_stack(
-        flat_name, ind=ind_flat, digit=5, slc=(sino, None))
+        flat_name, ind=ind_flat, slc=(sino, None))
     dark = dxreader.read_tiff_stack(
-        dark_name, ind=ind_dark, digit=5, slc=(sino, None))
+        dark_name, ind=ind_dark, slc=(sino, None))
     return tomo, flat, dark
 
 
@@ -398,7 +406,7 @@ def read_aps_1id(fname, ind_tomo=None, proj=None, sino=None):
     # File definitions.
     fname = os.path.abspath(fname)
     _fname = fname + '000001.tif'
-    log_file = os.path.dirname(fname) + os.path.sep + 'TomoStillScan.dat'
+    log_file = fname + 'TomoStillScan.dat'
 
     # Read APS 1-ID log file data
     contents = open(log_file, 'r')
@@ -421,14 +429,16 @@ def read_aps_1id(fname, ind_tomo=None, proj=None, sino=None):
 
     if ind_tomo is None:
         ind_tomo = list(range(prj_start, prj_start + nprj))
+    if proj is not None:
+        ind_tomo = ind_tomo[slice(*proj)]
     ind_flat = list(range(flat_start, flat_start + nflat))
     ind_dark = list(range(dark_start, dark_start + ndark))
     tomo = dxreader.read_tiff_stack(
-        _fname, ind=ind_tomo, digit=6, slc=(sino, proj))
+        _fname, ind=ind_tomo, slc=(sino, None))
     flat = dxreader.read_tiff_stack(
-        _fname, ind=ind_flat, digit=6, slc=(sino, None))
+        _fname, ind=ind_flat, slc=(sino, None))
     dark = dxreader.read_tiff_stack(
-        _fname, ind=ind_dark, digit=6, slc=(sino, None))
+        _fname, ind=ind_dark, slc=(sino, None))
     return tomo, flat, dark
 
 
@@ -491,6 +501,59 @@ def read_aps_7bm(fname, proj=None, sino=None):
     return tomo, theta
 
 
+def read_aps_8bm(image_directory, ind_tomo, ind_flat,
+                 image_file_pattern='image_00000.xrm',
+                 flat_file_pattern='ref_00000.xrm', proj=None, sino=None):
+    """
+    Read APS 8-BM tomography data from a stack of xrm files.
+
+    Parameters
+    ----------
+    image_directory : str
+        Path to data folder name without indices and extension.
+
+    ind_tomo : list of int
+        Indices of the projection files to read.
+
+    ind_flat : list of int
+        Indices of the flat field files to read.
+
+    image_file_pattern: string
+        Specify how the projection files are named.
+
+    flat_file_pattern: string
+        Specify how the flat reference files are named.
+
+    proj : {sequence, int}, optional
+        Specify projections to read. (start, end, step)
+
+    sino : {sequence, int}, optional
+        Specify sinograms to read. (start, end, step)
+
+    Returns
+    -------
+    ndarray
+        3D tomographic data.
+
+    ndarray
+        3D flat field data.
+
+    dictionary
+        Image set metadata.
+    """
+    image_directory = os.path.abspath(image_directory)
+    tomo_name = os.path.join(image_directory, 'radios', image_file_pattern)
+    flat_name = os.path.join(image_directory, 'flats', flat_file_pattern)
+    if proj is not None:
+        ind_tomo = ind_tomo[slice(*proj)]
+    tomo, metadata = dxreader.read_xrm_stack(
+        tomo_name, ind=ind_tomo, slc=(sino, None))
+
+    flat, _ = dxreader.read_xrm_stack(
+        flat_name, ind=ind_flat, slc=(sino, None))
+    return tomo, flat, metadata
+
+
 def read_aps_13bm(fname, format, proj=None, sino=None):
     """
     Read APS 13-BM standard data format.
@@ -514,9 +577,9 @@ def read_aps_13bm(fname, format, proj=None, sino=None):
     ndarray
         3D tomographic data.
     """
-    if format is 'spe':
+    if format == 'spe':
         tomo = dxreader.read_spe(fname, slc=(None, sino))
-    elif format is 'netcdf4':
+    elif format == 'netcdf4':
         tomo = dxreader.read_netcdf4(fname, 'array_data', slc=(proj, sino))
     return tomo
 
@@ -551,21 +614,21 @@ def read_aps_13id(
     return tomo
 
 
-def read_aps_26id(fname, ind_tomo, ind_flat, proj=None, sino=None):
+def read_aps_26id(image_directory, ind_tomo, ind_flat,
+                  image_file_pattern='image_00000.xrm',
+                  flat_file_pattern='ref_00000.xrm', proj=None, sino=None):
     """
     Read APS 26-ID tomography data from a stack of xrm files.
-    Note: file are renamed as 
-    radios/image_00000.xrm and flats/image_00000.xrm
-    
+
     Parameters
     ----------
     fname : str
         Path to data folder name without indices and extension.
 
-    ind_tomo : list of int, optional
+    ind_tomo : list of int
         Indices of the projection files to read.
 
-    ind_flat : list of int, optional
+    ind_flat : list of int
         Indices of the flat field files to read.
 
     proj : {sequence, int}, optional
@@ -581,19 +644,15 @@ def read_aps_26id(fname, ind_tomo, ind_flat, proj=None, sino=None):
 
     ndarray
         3D flat field data.
+
+    dictionary
+        Image set metadata.
     """
-    fname = os.path.abspath(fname)
-    tomo_name = os.path.join(fname, 'radios', 'image_00000.xrm')
-    flat_name = os.path.join(fname, 'flats', 'image_00000.xrm')
-
-    tomo = dxreader.read_xrm_stack(
-        tomo_name, ind=ind_tomo, digit=5, slc=(sino, proj))
-    flat = dxreader.read_xrm_stack(
-        flat_name, ind=ind_flat, digit=5, slc=(sino, None))
-    return tomo, flat
+    return read_aps_8bm(image_directory, ind_tomo, ind_flat,
+                        image_file_pattern, flat_file_pattern, proj, sino)
 
 
-def read_aps_32id(fname, exchange_rank=0, proj=None, sino=None):
+def read_aps_32id(fname, exchange_rank=0, proj=None, sino=None, dtype=None):
     """
     Read APS 32-ID standard data format.
 
@@ -604,7 +663,7 @@ def read_aps_32id(fname, exchange_rank=0, proj=None, sino=None):
 
     exchange_rank : int, optional
         exchange_rank is added to "exchange" to point tomopy to the data
-        to recontruct. if rank is not set then the data are raw from the
+        to reconstruct. if rank is not set then the data are raw from the
         detector and are located under exchange = "exchange/...", to process
         data that are the result of some intemedite processing step then
         exchange_rank = 1, 2, ... will direct tomopy to process
@@ -616,6 +675,9 @@ def read_aps_32id(fname, exchange_rank=0, proj=None, sino=None):
     sino : {sequence, int}, optional
         Specify sinograms to read. (start, end, step)
 
+    dtype : numpy datatype, optional
+        Convert data to this datatype on read if specified.    
+
     Returns
     -------
     ndarray
@@ -626,6 +688,9 @@ def read_aps_32id(fname, exchange_rank=0, proj=None, sino=None):
 
     ndarray
         3D dark field data.
+
+    ndarray
+        1D theta in radian.
     """
     if exchange_rank > 0:
         exchange_base = 'exchange{:d}'.format(int(exchange_rank))
@@ -635,10 +700,19 @@ def read_aps_32id(fname, exchange_rank=0, proj=None, sino=None):
     tomo_grp = '/'.join([exchange_base, 'data'])
     flat_grp = '/'.join([exchange_base, 'data_white'])
     dark_grp = '/'.join([exchange_base, 'data_dark'])
-    tomo = dxreader.read_hdf5(fname, tomo_grp, slc=(proj, sino))
-    flat = dxreader.read_hdf5(fname, flat_grp, slc=(None, sino))
-    dark = dxreader.read_hdf5(fname, dark_grp, slc=(None, sino))
-    return tomo, flat, dark
+    theta_grp = '/'.join([exchange_base, 'theta'])
+    tomo = dxreader.read_hdf5(fname, tomo_grp, slc=(proj, sino), dtype=dtype)
+    flat = dxreader.read_hdf5(fname, flat_grp, slc=(None, sino), dtype=dtype)
+    dark = dxreader.read_hdf5(fname, dark_grp, slc=(None, sino), dtype=dtype)
+    theta = dxreader.read_hdf5(fname, theta_grp, slc=None)
+
+    if (theta is None):
+        theta_size = dxreader.read_dx_dims(fname, 'data')[0]
+        logger.warn('Generating "%s" [0-180] deg angles for missing "exchange/theta" dataset' % (str(theta_size)))
+        theta = np.linspace(0. , np.pi, theta_size)
+    else:
+        theta = theta * np.pi / 180.
+    return tomo, flat, dark, theta
 
 
 def read_aus_microct(fname, ind_tomo, ind_flat, ind_dark, proj=None, sino=None):
@@ -681,9 +755,14 @@ def read_aus_microct(fname, ind_tomo, ind_flat, ind_dark, proj=None, sino=None):
     flat_name = os.path.join(fname, 'BG__BEFORE_00.tif')
     dark_name = os.path.join(fname, 'DF__BEFORE_00.tif')
 
-    tomo = dxreader.read_tiff_stack(tomo_name, ind=ind_tomo, digit=4, slc=(sino, proj))
-    flat = dxreader.read_tiff_stack(flat_name, ind=ind_flat, digit=2, slc=(sino, None))
-    dark = dxreader.read_tiff_stack(dark_name, ind=ind_dark, digit=2, slc=(sino, None))
+    if proj is not None:
+        ind_tomo = ind_tomo[slice(*proj)]
+    tomo = dxreader.read_tiff_stack(
+        tomo_name, ind=ind_tomo, slc=(sino, None))
+    flat = dxreader.read_tiff_stack(
+        flat_name, ind=ind_flat, slc=(sino, None))
+    dark = dxreader.read_tiff_stack(
+        dark_name, ind=ind_dark, slc=(sino, None))
     return tomo, flat, dark
 
 
@@ -724,7 +803,7 @@ def read_esrf_id19(fname, proj=None, sino=None):
     return tomo, flat, dark
 
 
-def read_diamond_l12(fname, ind_tomo):
+def read_diamond_l12(fname, ind_tomo, proj=None):
     """
     Read Diamond Light Source L12 (JEEP) standard data format.
 
@@ -736,6 +815,9 @@ def read_diamond_l12(fname, ind_tomo):
     ind_tomo : list of int
         Indices of the projection files to read.
 
+    proj : {sequence, int}, optional
+        Specify projections to read. (start, end, step)
+
     Returns
     -------
     ndarray
@@ -748,8 +830,10 @@ def read_diamond_l12(fname, ind_tomo):
     tomo_name = os.path.join(fname, 'im_001000.tif')
     flat_name = os.path.join(fname, 'flat_000000.tif')
     ind_flat = list(range(0, 1))
-    tomo = dxreader.read_tiff_stack(tomo_name, ind=ind_tomo, digit=6)
-    flat = dxreader.read_tiff_stack(flat_name, ind=ind_flat, digit=6)
+    if proj is not None:
+        ind_tomo = ind_tomo[slice(*proj)]
+    tomo = dxreader.read_tiff_stack(tomo_name, ind=ind_tomo)
+    flat = dxreader.read_tiff_stack(flat_name, ind=ind_flat)
     return tomo, flat
 
 
@@ -793,12 +877,14 @@ def read_elettra_syrmep(
     tomo_name = os.path.join(fname, 'tomo_0001.tif')
     flat_name = os.path.join(fname, 'flat_1.tif')
     dark_name = os.path.join(fname, 'dark_1.tif')
+    if proj is not None:
+        ind_tomo = ind_tomo[slice(*proj)]
     tomo = dxreader.read_tiff_stack(
-        tomo_name, ind=ind_tomo, digit=4, slc=(sino, proj))
+        tomo_name, ind=ind_tomo, slc=(sino, None))
     flat = dxreader.read_tiff_stack(
-        flat_name, ind=ind_flat, digit=1, slc=(sino, None))
+        flat_name, ind=ind_flat, slc=(sino, None))
     dark = dxreader.read_tiff_stack(
-        dark_name, ind=ind_dark, digit=1, slc=(sino, None))
+        dark_name, ind=ind_dark, slc=(sino, None))
     return tomo, flat, dark
 
 
@@ -881,12 +967,14 @@ def read_petraIII_p05(
         fname, 'scan_0001', 'ccd', 'pco01', 'ccd_0000.tif')
     dark_name = os.path.join(
         fname, 'scan_0000', 'ccd', 'pco01', 'ccd_0000.tif')
+    if proj is not None:
+        ind_tomo = ind_tomo[slice(*proj)]
     tomo = dxreader.read_tiff_stack(
-        tomo_name, ind=ind_tomo, digit=4, slc=(sino, proj))
+        tomo_name, ind=ind_tomo, slc=(sino, None))
     flat = dxreader.read_tiff_stack(
-        flat_name, ind=ind_flat, digit=4, slc=(sino, None))
+        flat_name, ind=ind_flat, slc=(sino, None))
     dark = dxreader.read_tiff_stack(
-        dark_name, ind=ind_dark, digit=4, slc=(sino, None))
+        dark_name, ind=ind_dark, slc=(sino, None))
     return tomo, flat, dark
 
 
@@ -946,13 +1034,15 @@ def read_sls_tomcat(fname, ind_tomo=None, proj=None, sino=None):
 
     if ind_tomo is None:
         ind_tomo = list(range(proj_start, proj_end))
+    if proj is not None:
+        ind_tomo = ind_tomo[slice(*proj)]
     ind_flat = list(range(flat_start, flat_end))
     ind_dark = list(range(dark_start, dark_end))
     tomo = dxreader.read_tiff_stack(
-        _fname, ind=ind_tomo, digit=4, slc=(sino, proj))
+        _fname, ind=ind_tomo, slc=(sino, None))
     flat = dxreader.read_tiff_stack(
-        _fname, ind=ind_flat, digit=4, slc=(sino, None))
+        _fname, ind=ind_flat, slc=(sino, None))
     dark = dxreader.read_tiff_stack(
-        _fname, ind=ind_dark, digit=4, slc=(sino, None))
+        _fname, ind=ind_dark, slc=(sino, None))
 
     return tomo, flat, dark
